'use strict';

const validationTests = {
  onlyNameChars: (value) => /^[A-Za-zÀ-ÖØ-öø-ÿěščřžďťňůĚŠČŘŽĎŤŇŮ -]*$/.test(value),
  startsAndEndsWithLetter: (value) =>
    /^(([A-Za-zÀ-ÖØ-öø-ÿěščřžďťňůĚŠČŘŽĎŤŇŮ].*[A-Za-zÀ-ÖØ-öø-ÿěščřžďťňůĚŠČŘŽĎŤŇŮ])|[A-Za-zÀ-ÖØ-öø-ÿěščřžďťňůĚŠČŘŽĎŤŇŮ])?$/.test(value),
  noWhitespace: (value) => /^\S*$/.test(value),
  hasNumber: (value) => /^\S*[0-9]\S*$/.test(value),
  hasLowercaseLetter: (value) => /^\S*[a-z]\S*$/.test(value),
  hasUppercaseLetter: (value) => /^\S*[A-Z]\S*$/.test(value),
  hasSpecialChar: (value) => /^\S*[^0-9a-zA-ZÀ-ÖØ-öø-ÿěščřžďťňůĚŠČŘŽĎŤŇŮ\s]\S*$/.test(value),
  validEmail: (value) =>
    /^(([^<>()[\]\.,;:\s@\"]+(\.[^<>()[\]\.,;:\s@\"]+)*)|(\".+\"))@(([^<>()[\]\.,;:\s@\"]+\.)+[^<>()[\]\.,;:\s@\"]{2,})$/i.test(value),
};

class FormValidation {
  constructor(form, validations, successHeading, successMessage) {
    this.form = form;
    this.fields = form.querySelectorAll('.field');
    this.validations = validations;
    this.successMessage = successMessage;
    this.successHeading = successHeading;
    this.formStatus = document.getElementById('formStatus');

    // Save hint messages
    this.hintMessages = {};
    this.fields.forEach((field) => {
      this.hintMessages[field.id] = field.parentElement.querySelector('.hint');
    });

    // Show password behavior
    const passwordField = document.getElementById('password');
    document.getElementById('showPassword').addEventListener('change', (event) => {
      password.setAttribute('type', (event.target.checked ? 'text' : 'password'));
    });

    this.form.addEventListener('submit', this.validate.bind(this));
  }
  validate(event) {
    event.preventDefault();
    this.fields.forEach((field) => {
      // Find out invalid rules
      let shownErrors = [];
      const fieldRules = this.validations[field.id].rules;
      if (fieldRules) {
        for (let ruleName in fieldRules) {
          const ruleValue = fieldRules[ruleName];

          // Test required rule
          const isCheckbox = (field.getAttribute('type') === 'checkbox');
          if ((ruleName === 'required') && ruleValue && ((isCheckbox && (field.checked === false)) || (!isCheckbox && (field.value.length === 0)))) {
            shownErrors.push('required');

            // Test minLength rule
          } else if ((ruleName === 'minLength') && (field.value.length !== 0) && (field.value.length < ruleValue)) {
            shownErrors.push('minLength');

            // Test maxLength rule
          } else if ((ruleName === 'maxLength') && (field.value.length !== 0) && (field.value.length > ruleValue)) {
            shownErrors.push('maxLength');

            // Test regex rules
          } else if ((field.value.length !== 0) && ruleName === 'regexes') {
            ruleValue.forEach((testName) => {
              if (!validationTests[testName](field.value)) {
                shownErrors.push(testName);
              }
            });
          }
        }
      }

      const errorsId = `${field.id}-errors`;
      let errorsContainer = document.getElementById(errorsId);

      // Remove previous error messages if exist
      if (errorsContainer) {
        errorsContainer.parentElement.removeChild(errorsContainer);
      }

      // Create and append error messages, possibly remove the hint message and set ARIA attributes for the field
      if (shownErrors.length > 0) {
        const shownErrorsIds = [];
        errorsContainer = document.createElement('div');
        errorsContainer.setAttribute('id', errorsId);
        errorsContainer.setAttribute('class', 'errors');
        errorsContainer.innerHTML = this.validations[field.id].getErrorMessages(shownErrors);
        field.parentElement.appendChild(errorsContainer);
        this.hintMessages[field.id]?.remove();

        // Set aria-describedby to refer to the shown errors
        errorsContainer.querySelectorAll('.error').forEach((error) => {
          shownErrorsIds.push(error.id);
        });
        field.setAttribute('aria-describedby', shownErrorsIds.join(' '));

        // Set aria-invalid
        field.setAttribute('aria-invalid', 'true');
        } else {
          field.setAttribute('aria-describedby', undefined);
        field.setAttribute('aria-invalid', undefined);
      }
    });

    // Focus the first invalid field if it exists
    const firstErrorField = this.form.querySelector('[aria-invalid="true"]');
    if (firstErrorField) {
      // The timeout is needed for focus to work reliably
      window.setTimeout(() => {
        firstErrorField.focus();
      }, 100);

      // Hide the form, replace h1 content and insert the success message otherwise
    } else {
      document.querySelector('h1').innerText = this.successHeading;
      this.form.remove();
      this.formStatus.innerHTML = this.successMessage;
    }
  }
}

window.addEventListener('load', () => {
  const form = document.getElementById('registration-form');
  const validations = {
    fullName: {
      rules: {
        required: true,
        minLength: 2,
        maxLength: 50,
        regexes: [
          'onlyNameChars',
          'startsAndEndsWithLetter',
        ],
      },
      getErrorMessages: (errors) => {
        let output = '';
        if (errors.includes('required')) {
          output += '<p id="fullNameRequiredError" class="error">Jméno je povinné.</p>';
        } else {
          output += `<p id="fullNameInvalidError" class="error">Jméno nesplňuje tyto požadavky: </p>
  <ul>` +
            ((errors.includes('minLength') || errors.includes('maxLength')) ? '<li id="fullNameLengthError" class="error">Musí obsahovat nejméně 2 znaky, nejvíce však 50 znaků.</li>' : '') +
            ((errors.includes('onlyNameChars')) ? '<li id="fullNameOnlyNameCharsError" class="error">Musí obsahovat pouze malá písmena, velká písmena, mezery nebo pomlčky.</li>' : '') +
            ((errors.includes('startsAndEndsWithLetter')) ? '<li id="fullNameStartsAndEndsWithLetterError" class="error">Musí začínat a končit písmenem. </li>' : '') +
            '</ul>';
        }
        return output;
      },
    },
    nickname: {
      rules: {
        minLength: 2,
        maxLength: 20,
        regexes: [
          'onlyNameChars',
          'startsAndEndsWithLetter',
        ],
      },
      getErrorMessages: (errors) => {
        const output = `<p id="nicknameInvalidError" class="error">Přezdívka nesplňuje tyto požadavky: </p>
        <ul>` +
          ((errors.includes('minLength') || errors.includes('maxLength')) ? '<li id="nicknameLengthError" class="error">Musí obsahovat nejméně 2 znaky, nejvíce však 20 znaků.</li>' : '') +
          ((errors.includes('onlyNameChars')) ? '<li id="nicknameOnlyNameCharsError" class="error">Musí obsahovat pouze malá písmena, velká písmena, mezery nebo pomlčky.</li>' : '') +
          ((errors.includes('startsAndEndsWithLetter')) ? '<li id="nicknameStartsAndEndsWithLetterError" class="error">Musí začínat a končit písmenem. </li>' : '') +
          '</ul>';
        return output;
      },
    },
    email: {
      rules: {
        required: true,
        regexes: [
          'validEmail',
        ],
      },
      getErrorMessages: (errors) => {
        let output = '';
        if (errors.includes('required')) {
          output += '<p id="emailRequiredError" class="error">E-mail je povinný.</p>';
        } else if (errors.includes('validEmail')) {
          output += '<li id="emailCharsError" class="error">E-mail musí být platnou emailovou adresou, jako například jmeno@example.com.</li>';
        }
        return output;
      },
    },
    password: {
      rules: {
        required: true,
        minLength: 8,
        maxLength: 30,
        regexes: [
          'hasLowercaseLetter',
          'hasUppercaseLetter',
          'hasSpecialChar',
          'hasNumber',
          'noWhitespace',
        ],
      },
      getErrorMessages: (errors) => {
        let output = '';
        if (errors.includes('required')) {
          output += `<p id="passwordInvalidError" class="error">Heslo nesplňuje tyto požadavky: </p>
          <ul>
          <li id="passwordLengthError" class="error">Musí obsahovat nejméně 8 znaků, nejvíce však 30 znaků.</li>
          <li id="passwordCharsError" class="error">Musí obsahovat malé písmeno, velké písmeno, číslo, speciální znak a žádnou mezeru.</li>
          </ul>`;
        } else {
          output += `<p id="passwordInvalidError" class="error">Heslo nesplňuje tyto požadavky: </p>
  <ul>` +
            ((errors.includes('minLength') || errors.includes('maxLength')) ? '<li id="passwordLengthError" class="error">Musí obsahovat nejméně 8 znaků, nejvíce však 30 znaků.</li>' : '') +
            ((errors.includes('hasLowercaseLetter') || errors.includes('hasUppercaseLetter') || errors.includes('hasSpecialChar') || errors.includes('hasNumber') || errors.includes('noWhitespace')) ? '<li id="passwordCharsError" class="error">Musí obsahovat malé písmeno, velké písmeno, číslo, speciální znak a žádnou mezeru.</li>' : '') +
            '</ul>';
        }
        return output;
      },
    },
    acceptTerms: {
      rules: {
        required: true,
      },
      getErrorMessages: (errors) => {
        let output = '';
        if (errors.includes('required')) {
          output += '<p id="acceptTermsRequiredError" class="error">Pro registraci je nutné souhlasit s podmínkami a ujednáním.</p>';
        }
        return output;
      },
    },
  };

  // Successful form submission heading and message
  const successHeading = 'Registrace byla úspěšná';
  const successMessage = '<p>Děkujeme, registrace proběhla úspěšně.</p>';

  const validation = new FormValidation(form, validations, successHeading, successMessage);
});
